#!/usr/local/gnu/bin/perl5.004
##==============================================================================
## File Name         : fix_ver.pl
## Date Created      : 9/11/2000
## Date Last Revised : 10/19/2000
## Organization      : IP Solutions Division / System Interface Group
## Description       : Append/insert sentence(s) in a file
##------------------------------------------------------------------------------
## Usage             : $0
##------------------------------------------------------------------------------
## Arguments         : -
##------------------------------------------------------------------------------
## Dependent Files   : $FILE_R
##------------------------------------------------------------------------------
## Revision History  :
## 1.4 [Eric - 6/25/2001]
##   Updated to reflect new design flows.
##
## 1.3 [Eric - 10/19/2000]
##   Updated to work for v3.0 core.
##
## 1.2 [Eric - 9/15/2000]
##   Updated to incorporate into source generation flow.
##
## 1.1 [Matt - 9/14/2000]
##   UpdateProgress($)
##    modification of interface: now modify a reference on $progress
##
## 1.0 [Matt - 9/11/2000]
##   Initial release
##==============================================================================


## Prototypes ##################################################################

sub Progress($);
sub UpdateProgress($);


## Data file names #############################################################

local $FILE_R  = "sdram_top_sim.v";
local $FILE_W  = "fixed.v";


## Constant declarations #######################################################

local $lookUp0 = { string  => "output",
                   append  => "// synthesis syn_edif_bit_format = \"%u<%i>\"\n".
                              "// synthesis syn_edif_scalar_format = \"%u\"\n" .
                              "// synthesis syn_noclockbuf = 1\n"              .
                              "// synthesis syn_hier = \"hard\"\n"             .
                              "// synthesis syn_black_box\n"};

local $lookUp1 = { string  => "wire",
                   append  => "// synopsys translate_off\n"};

local $lookUp2 = { string  => "endmodule",
                   append  => "// synopsys translate_on\n"};


## MAIN ########################################################################

my @lookUp    = ($lookUp0, $lookUp1, $lookUp2);
my $index     = 0;
my $line      = "";
my $progress  = CreateProgress($#lookUp);

open(FILE_R, "$FILE_R")  || die("Can't open $FILE_R");
open(FILE_W, ">$FILE_W") || die("Can't open $FILE_W");

while(<FILE_R>)
{
  ## Copy blindly comments
  if (/^\s*--/) {
    print FILE_W ($_);
  }

  elsif (/$lookUp[$index]->{string}/) {

    UpdateProgress(\$progress);

    if (exists($lookUp[$index]->{append})) {
      print FILE_W ($lookUp[$index]->{append});
    }

    print FILE_W ($_);

    if (exists($lookUp[$index]->{insert})) {
      print FILE_W ($lookUp[$index]->{insert});
    }

    ## Seek next look up
    if ($index < $#lookUp) {
      $index++;
    }
  }

  else {
    ## Copy non matching line
    print FILE_W ($_);
  }
} 

print("\Closing files\n");
close(FILE_W) || die("Can't close $FILE_W");
close(FILE_R) || die("Can't close $FILE_R");


## Subroutines #################################################################

##------------------------------------------------------------------------------
## Function: CreateProgress
## Purpose : Display the number of execution step
## IN      : -
## OUT     : $progress
sub CreateProgress($) {
  my $nb_step = shift;
  my $i;

  my $progress = "[";
  for($i = 0; $i <= $nb_step; $i++) {
    $progress .= ".";
  }
  $progress .= "]";
  print STDOUT ("$progress");

  return($progress);
}

##------------------------------------------------------------------------------
## Function: UpdateProgress
## Purpose : Check a step as done
## INOUT   : \$progress
sub UpdateProgress($) {
  $ref_progress = $_[0];
  
  $$ref_progress =~ s/\./\*/;
  print STDOUT ("\r$$ref_progress");
  if ($$ref_progress !~ /\./) {
    print STDOUT ("\n");
  }
}
