/*--

-- File name 	: 	UART_CORE.v

--             :

-- Title       :  UART CORE

--             :

-- Purpose     :  

--					:

--             :

--             :

--             :

-- Created On  : 03/20/02

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: 

--             : 

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx SPARTAN/VIRTEX FPGA

--   				:

-- Developers  : ERIC ZHANG

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | EZ  			| First Written						| 

-- ------------------------------------------------------------------------*/

`timescale 1 ns/1 ps





module UART_CORE (

   RESET,

   CLK,

   

   CTRL_REG,

   WEN_TXFIFO,

   WDATA_TXFIFO,

   REN_RXFIFO, 

   RDATA_RXFIFO,   

   FIFO_STATUS,

   RX_OF,

   

   SIN,

   SOUT

   );

   

input          RESET;

input          CLK;

input  [31:0]  CTRL_REG;

input          WEN_TXFIFO;

input  [7:0]   WDATA_TXFIFO;

input          REN_RXFIFO;

output [7:0]   RDATA_RXFIFO;

input          SIN;

output         SOUT; 

output [15:0]  FIFO_STATUS;

output         RX_OF;



//wire   [8:0]  rxfifo_level;  

wire        tx_pop;

wire        tx_empty;

wire [7:0]  tx_pop_data;   

wire        rx_push;

wire [7:0]  rx_push_data;

wire        tx_full;

wire        rx_empty;

wire [8:0]  rx_count;   



wire        baud_ce;

wire        lsr_ack;

wire        rbr_ack;

wire  [6:0] lcr;

wire  [4:0] mcr;

wire  [3:0] ier;

wire  [6:0] lsr_out;

wire        gnd;

wire        vcc;



assign FIFO_STATUS = {rx_count,4'b0,rx_empty, tx_full, tx_empty}; 



assign lsr_ack = lsr_out[5];

assign rbr_ack = lsr_out[0];

assign lcr = 'h3;

assign mcr = 'h0;

assign ier = 'h0;

assign gnd = 1'b0;

assign vcc = 1'b1;



UART_TXFIFO iUART_TXFIFO(

   .RESET            (RESET),

   .CLK              (CLK),

   .TX_PUSH          (WEN_TXFIFO),

   .TX_PUSH_DATA     (WDATA_TXFIFO),

   .TX_FULL          (tx_full),

   

   .TX_POP           (tx_pop),

   .TX_EMPTY         (tx_empty),

   .TX_POP_DATA      (tx_pop_data)

   );   



UART_RXFIFO iUART_RXFIFO(

   .RESET            (RESET),

   .CLK              (CLK),

   .RX_PUSH          (rx_push),

   .RX_PUSH_DATA     (rx_push_data),

   

   .RX_POP           (REN_RXFIFO),

   .RX_EMPTY         (rx_empty),

   .RX_POP_DATA      (RDATA_RXFIFO),

   .RX_OF            (RX_OF),

   .RX_COUNT         (rx_count)

   );   



XF_UART iXF_UART (

	// inputs

	.SIN		(SIN),		// Transmit Serial Data

	.TX_CE		(baud_ce),	// Transmit CLK enable

	.RX_CE		(baud_ce),	// Receive CLK enable

	.CTS_L		(vcc),	// Data Clear to Send

	.DSR_L		(vcc),   	// Data Set Ready

	.DCD_L		(vcc),   	// Data Carrier Detect

	.RI_L		(vcc),		// Ring Indicator

	.DL			(CTRL_REG[15:0]),// Delay Latch Register

	.LCR		(lcr),	// Line Control	Register

	.MCR		(mcr),	   	// Modem Control Register

	.IER		(ier),	   	// Interrupt Enable Register

	.THR		(tx_pop_data),	   	// Transmit Holding Register

	.FMODE_RX	(vcc),	// Transmit FIFO Mode

	.FMODE_TX	(vcc),	// Receive FIFO Mode

	.THR_CE		(gnd),  	// Transmit Holding Reg Clock Enable

	.FEMPTY		(tx_empty),	// FIFO Empty Flag

	.FFULL		(gnd), 	// FIFO Full Flag

	.RBR_ACK	(rbr_ack), 	// Receive Buffer Register Acknowledge

	.IIR_ACK	(gnd), 	// Interrupt Ident. Register Acknowledge

	.LSR_ACK	(lsr_ack), 	// Line Status Register Acknowledge

	.MSR_ACK	(gnd), 	// Modem Status Register Acknowledge

	.CLK		(CLK), 		// Master Clock

	.BRG_CE		(vcc),	// Master Clock enable

	.RST		(RESET),	 	// Master Reset (async)

	// outputs

	.SOUT		(SOUT),	  	// Receive Serial Data

	.BAUD_CE	(baud_ce),	// Baud Clock Enable

	.RTS_L		(), 	// Request to Send

	.DTR_L		(), 	// Data Terminal Ready

	.OUT1_L		(),	// Output bit 1

	.OUT2_L		(),	// Output bit 2

	.RBR		(rx_push_data),	// Receive Buffer Register

	.LSR		(lsr_out),	// Line Status Register

	.MSR		(),	// Modem Status Register

	.IIR		(),	// Interrupt Ident. Register

	.INTRPT		(),	// Interrupt Request

	.FREAD		(tx_pop),	// FIFO Read Enable

	.FWRITE		(rx_push)	// FIFO Write Enable

);

   



endmodule