/*--

-- File name 	: 	UART_TXFIFO.v

--             :

-- Title       :  UART TX FIFO (512 x 8 bits)

--             :

-- Purpose     :  buffering transmitter data

--					:

--             :

--             :

--             :

-- Created On  : 03/20/02

--             :

-- Comments    : Design done by Insight Design Services

--             :

--					:

--					:

--             :

-- Conventions :

--					: 

--             : 

--   				:

--   				:

--   				:

-- Limitations : For use in Xilinx SPARTAN2 FPGA

--   				:

-- Developers  : ERIC ZHANG

-- ----------------------------------------------------------------------

-- Revision History :

-- ----------------------------------------------------------------------

--   Version No:| Author    	| Changes Made: 						| Mod. Date:

--     v1.0     | EZ  			| First Written						| 

-- ------------------------------------------------------------------------*/

`timescale 1 ns/1 ps





module UART_TXFIFO (

   RESET,

   CLK,

   TX_PUSH,

   TX_PUSH_DATA,

   TX_FULL,

   

   TX_POP,

   TX_EMPTY,

   TX_POP_DATA

   );   

   

input          RESET;

input          CLK;

input          TX_PUSH;

input  [7:0]   TX_PUSH_DATA;

output         TX_FULL;

   

input          TX_POP;

output         TX_EMPTY;

output [7:0]   TX_POP_DATA;  





//------------------------------------------------------------------------------

//  Variable declarations

//------------------------------------------------------------------------------

reg  [8:0]  addr_wr;

wire [8:0]  next_addr_wr;

reg  [8:0]  addr_rd;

wire [8:0]  next_addr_rd;



reg      FULL, EMPTY;



wire        valid_push;

wire        valid_pop;



wire [7:0] pZero8;

wire        vcc;

wire        gnd;



assign pZero8 = 'h0;

assign vcc     = 1;

assign gnd     = 0;



//------------------------------------------------------------------------------

//  Begin Module

//------------------------------------------------------------------------------

assign TX_FULL = FULL;

assign TX_EMPTY = EMPTY;



assign valid_push = TX_PUSH & ~FULL;

assign valid_pop  = TX_POP & ~EMPTY;



assign next_addr_wr = addr_wr + 1;

assign next_addr_rd = addr_rd + 1;



//-------------------------------------------------

//---------------- FIFO status flag  --------------

//-------------------------------------------------

always @ (posedge RESET or posedge CLK)

   if (RESET) FULL <= 0;

   else if (TX_PUSH && (next_addr_wr == addr_rd)) FULL <= 1;

   else if (addr_wr != addr_rd) FULL <= 0;

      

always @ (posedge RESET or posedge CLK)

   if (RESET) EMPTY <= 1;

   else if (TX_POP && (next_addr_rd == addr_wr)) EMPTY <= 1;

   else if (addr_wr != addr_rd) EMPTY <= 0;

      

//-------------------------------------------------

//------------ FIFO write/read address -----------

//-------------------------------------------------

always @ (posedge RESET or posedge CLK)

   if (RESET) addr_wr <= 'b0;

   else if (valid_push) addr_wr <= next_addr_wr;

      

// FIFO read address 

always @ (posedge RESET or posedge CLK)

   if (RESET) addr_rd <= 'b0;

   else if (valid_pop) addr_rd <= next_addr_rd;

      

//-------------------------------------------------

//--------- BLOCK RAM instances  --------------

//-------------------------------------------------



RAMB4_S8_S8  iUART_TXFIFO_BRAM

       (.DIA(TX_PUSH_DATA),

        .DIB(pZero8),

        .ENA(vcc),

        .ENB(vcc),

        .WEA(valid_push),

        .WEB(gnd),

        .RSTA(RESET),

        .RSTB(RESET),

        .CLKA(CLK),

        .CLKB(CLK),

        .ADDRA(addr_wr),

        .ADDRB(addr_rd),

        .DOA(),

        .DOB(TX_POP_DATA)

       ); 



endmodule