----------------------------------------------------------------------------------------------------

--             :

-- File name   :  PCI_ARBITER.VHD

--             :

-- Title       :

--             :

-- Comments    :

--             :

--             :

-- Conventions : - Port names are 'UPPER' case.

--             : - Internal signals and variables are 'lower' case.

--             : - Active low signals are identified with '_n' or '_N'.

--             : - states begin with lower case 's', rest upper case

--             : - aliases begin with lower case 'a', rest upper case

--             : - instances begin with lower case 'i', rest upper case

--             : - internal versions of Output signals begin with lower

--             :   case 'w', rest upper case

--             :

--             : Tab setting = 3

--             : Right Margin = 100

--             :

-- Limitations :

--             :

-- Developers  :  Majid Kalsi

--             :

----------------------------------------------------------------------------------------------------

-- Revision History :

----------------------------------------------------------------------------------------------------

-- $log$

----------------------------------------------------------------------------------------------------



library ieee;

use ieee.std_logic_1164.all;

use ieee.std_logic_unsigned.all;

use ieee.std_logic_arith.all;

use ieee.std_logic_textio.all;

use std.textio.all;

use work.utils_pkg.all;

--use work.defines_pkg.all;





----------------------------------------------------------------------------------------------------

-- Entity Declaration

----------------------------------------------------------------------------------------------------

entity pci_arbiter is

generic( SIZE : positive := 2);

port(

   CLK      : in  std_logic;

   REQ_N    : in  std_logic_vector((SIZE-1) downto 0);

   GNT_N    : out std_logic_vector((SIZE-1) downto 0);

   FRAME_N  : in  std_logic;

   IRDY_N   : in  std_logic

);

end pci_arbiter;



----------------------------------------------------------------------------------------------------

-- Architecture Declaration

----------------------------------------------------------------------------------------------------

architecture pci_arbiter_arch of pci_arbiter is





----------------------------------------------------------------------------------------------------

-- define constants

----------------------------------------------------------------------------------------------------

constant Tgnt         : time :=  6.0 ns;



----------------------------------------------------------------------------------------------------

-- define signals

----------------------------------------------------------------------------------------------------

signal reg_gnt_n : std_logic_vector((SIZE-1) downto 0) := (others => '1');





----------------------------------------------------------------------------------------------------

begin    -- begin Architecture

----------------------------------------------------------------------------------------------------



-- assign output

GNT_N <= reg_gnt_n after Tgnt;





process

   variable i, j : natural;

   variable request : boolean;

	variable last_time : std_logic_vector((SIZE-1) downto 0) := (others => '0');

begin



   wait until rising_edge(CLK);



   -- check if requests active

	i := 0;

   request := false;

   while (i < SIZE) and (not request) loop

      if To_X01(REQ_N(i)) = '0' then

         request := true;

      else

         request := false;

      end if;

      i:= i + 1;

   end loop;



   -- if requests active

	if request then

      -- figure out which was granted last time

      i := 0;

   	while (i < (SIZE-1)) and (last_time(i) = '0') loop

         i := i + 1;

      end loop;



      -- if none were granted, ie reset condition

      -- or 

		-- if rollover condition

      if (i = SIZE) or (i = (SIZE-1)) then

         j := 0;

      else

         j := i + 1;

      end if;



      -- find next requesting device in round-robin

      while To_X01(REQ_N(j)) = '1' loop

         if j = (SIZE-1) then

            j := 0;

         else

            j := j + 1;

         end if;

      end loop;



      -- give GRANT to requesting device

      reg_gnt_n(j) <= '0';

      last_time := (others => '0');

      last_time(j) := '1';



      -- wait for frame to assert

      while To_X01(FRAME_N) = '1' loop

   		wait until rising_edge(CLK);

      end loop;



      -- remove grant

      reg_gnt_n <= (others => '1');



      -- wait for IRDY_N to assert

      if To_X01(IRDY_N) = '1' then

         wait until To_X01(IRDY_N) = '0';

      end if;



      -- wait for IRDY to de-assert

      if To_X01(IRDY_N) = '0' then

         wait until To_X01(IRDY_N) = '1';

      end if;

   end if;

end process;





----------------------------------------------------------------------------------------------------

end pci_arbiter_arch;

----------------------------------------------------------------------------------------------------

