//--------------------------------------------------------------------------------------------------

//             :

// File name   : PCI_DEVICE_INITIATOR_S2PCI.V

//             :

// Title       : S2PCI specific PCI device Initiator Tasks

//             :

// Comments    :

//             :

//             :

// Conventions : - Port names are 'UPPER' case.

//             : - Internal signals and variables are 'lower' case.

//             : - Active low signals are identified with '_n' or '_N'.

//             : - instances begin with lower case 'i', rest upper case

//             : - parameters begin with lower case 'p', rest upper case

//             : - tasks begin with lower case 't', rest upper case

//             :

//             : Tab setting = 3

//             : Right Margin = 100

//             :

// Limitations : For use in testbench only

//             :

// Developers  : Duncan Allen

//             :

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//   Version No | Author    | Changes Made:                                            | Mod. Date:

//

//     v1.0     | da        | First written							          					| 27/08/01

//--------------------------------------------------------------------------------------------------



// This file contains PCI tasks to test the S2PCI design



task tXFER_STATUS_S2PCI;

   input [3:0] 	write_read;

   output [3:0]   return_stat;



   integer devsel_cnt;

   integer trdy_cnt;



   begin

      assign devsel_cnt = 0;

      assign trdy_cnt = 0;



      // While DEVSEL_N is unasserted loop, incrementing devsel_cnt each clk cycle

      // If DEVSEL_N is not asserted in 10 clock cycles then give up

      while (DEVSEL_N && (devsel_cnt <0))

         begin

         @(posedge CLK)

         devsel_cnt = devsel_cnt + 1;

         end



      // A similar loop waiting for TRDY_N or STOP_N to be asserted, trdy_cnt is incremented on each

      // clock cycle. If trdy_cnt reaches 16 give up and if dvsel_cnt had reached 10 then this

      // loop is skipped altogether

      while (TRDY_N && STOP_N && (trdy_cnt < 16) && (devsel_cnt < 10))

         begin

         @(posedge CLK)

         trdy_cnt = trdy_cnt + 1;

         end



      if (devsel_cnt < 10)    // If the device responded

         begin

         if (trdy_cnt <= 16)  // If it was ready in time

            begin

            if (!TRDY_N && STOP_N && FRAME_N)      // Normal initiator stop

               begin

               return_stat <= 4'b0001;

               if (write_read == 4'b1111)

                  $display ("STIM --> UUT : Normal Termination, Data Transferred");

               else

                  $display ("STIM <-- UUT : Normal Termination, Data Transferred");

               end

            else if (!TRDY_N && STOP_N && !FRAME_N)   //Normal data transfer Initiator waiting for more

               begin

               return_stat = 4'b0000;

               if (write_read == 4'b1111)

                  $display ("STIM --> UUT : Data Transferred, Transaction Continues");

               else

                  $display ("STIM <-- UUT : Data Transferred, Transaction Continues");

               end

            else if (!TRDY_N && !STOP_N)     // Target disconnect with Data

               begin

               return_stat = 4'b0010;

               if (write_read == 4'b1111)

                  $display ("STIM --> UUT : Disconnect, Data Transferred");

               else

                  $display ("STIM <-- UUT : Disconnect, Data Transferred");

               end

            else if (TRDY_N && !STOP_N)    // Target disconnect without Data

               begin

               return_stat = 4'b0011;

               if (write_read == 4'b1111)

                  $display ("STIM --> UUT : Retry, No Data Transferred");

               else

                  $display ("STIM <-- UUT : Retry, No Data Transferred");

               end

            else if (TRDY_N && !STOP_N && DEVSEL_N)   // Target Abort

               begin

               return_stat = 4'b0100;

               if (write_read == 4'b1111)

                  $display ("STIM --> UUT : Target Abort, No Data Transferred");

               else

                  $display ("STIM <-- UUT : Target Abort, No Data Transferred");

               end

            else if (TRDY_N && STOP_N)          // This should not happen

               begin

               return_stat = 4'b0101;

               $display (" ERROR : Check Transfer Procedure");

               end

            else

            	begin

            	return_stat <= 4'b0110;

            	$display (" ERROR : No Target Response");

            	end

      		end

      	else

         	begin

         	return_stat = 4'b0111;

         	$display (" ERROR : Master Abort");

         	end

      end

   end

endtask //tXFER_STATUS_S2PCI;











task tMEMORY_WRITE_8;

	input [31:0] 	START_ADDRESS;

	input [31:0] 	DATA_0;

	input [31:0] 	DATA_1;

	input [31:0] 	DATA_2;

	input [31:0] 	DATA_3;

	input [31:0] 	DATA_4;

	input [31:0] 	DATA_5;

	input [31:0]	DATA_6;

	input [31:0] 	DATA_7;

	output [7:0]    RETURN_STATUS;

   	

	reg [3:0]      	status_code;

   	reg [31:0]     	data [7:0];

	integer			number_of_items;

	integer			word_count;   // Then number of words that have been written



	begin

		$display ($time, " ns, Beginning write to address %hH", START_ADDRESS);

		number_of_items = 8; // For this task the number of items is fixed

		word_count = 1;

		data[0] = DATA_0;

		data[1] = DATA_1;

		data[2] = DATA_2;

		data[3] = DATA_3;

		data[4] = DATA_4;

		data[5] = DATA_5;

		data[6] = DATA_6;

		data[7] = DATA_7;

		@(posedge CLK) // Make sure everthing is synchronous



		// On the first clock cycle FRAME_N is asserted, the address is put on the AD bus and

		// the command for memory write is placed on CBE

		reg_frame_n <= 1'b0;

		reg_irdy_n <= 1'b1;

		reg_ad <= START_ADDRESS;

		reg_cbe <= 4'b0111;

		init_oe <= 1'b1;

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;



		@(posedge CLK)

		// On the second clock cycle FRAME_N remains asserted, CBE is set to allow all bytes to be read

		// The value of the first word to be written is placed on the bus

		// IRDY_N is asserted to indicate that the initiator is ready to transmit

		reg_frame_n <= 1'b0;

		reg_cbe <= 4'b0000;

		reg_ad <= data[0];

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;

		reg_irdy_n <= 1'b0;

      tXFER_STATUS_S2PCI ("1111", status_code);

      #1

      if (status_code == 4'b0000 || status_code == 4'b0010)

         $display ("Memory write #1 at Addr %h Data = %h", START_ADDRESS, DATA_0);

	  //Now write out the remainder of the data

	  if ((number_of_items != 1) && (status_code == 4'b000))		// If there is more than one item of data and the first write was sucessful

		  begin

		  while ((status_code == 4'b0000 || status_code == 4'b001) && word_count < number_of_items) // While there is data to write and the target has not disconnected

		  	begin

			// If this is the second last data item, disassert FRAME_N

			if (word_count == (number_of_items - 1))

				reg_frame_n = 1'b1;

			



			reg_ad = data[word_count];

			@(posedge CLK)

			tXFER_STATUS_S2PCI("1111", status_code);

			#1

			if (status_code == 4'b0000 || status_code == 4'b0010)

         		$display ("Memory write #%d at Addr %h Data = %h",word_count, START_ADDRESS, data[word_count]);



			word_count = word_count + 1;

		    end

		end



      // After the write has taken place, IRDY_N is deasserte, FRAME_N and CBE are tristated

      reg_irdy_n = 1'b1;

      reg_frame_n = 1'b1;

      ad_oe = 1'b0;

      cbe_oe = 1'b0;

      @(posedge CLK)



      init_oe <= 1'b0;

      @(posedge CLK);

	  RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase



   end

endtask //tMEMORY_WRITE_8





// This task will write N DATA words (N > 0)

// Each data will be increemented 1 from the start Data

task tMEMORY_WRITE_N;

	input [31:0] 	START_ADDRESS;

	input [31:0] 	START_DATA;

	input [31:0] 	BURST_SIZE;



	output [7:0]    RETURN_STATUS;



	reg [3:0]      	status_code;

   	reg [31:0]     	data [7:0];

	integer			number_of_items;

	integer			word_count;   // Then number of words that have been written



	begin

		$display ($time, " ns, Beginning write to address %hH", START_ADDRESS);

		number_of_items = BURST_SIZE;

		word_count = 1;



		@(posedge CLK) // Make sure everthing is synchronous



		// On the first clock cycle FRAME_N is asserted, the address is put on the AD bus and

		// the command for memory write is placed on CBE

		reg_frame_n <= 1'b0;

		reg_irdy_n <= 1'b1;

		reg_ad <= START_ADDRESS;

		reg_cbe <= 4'b0111;

		init_oe <= 1'b1;

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;



		@(posedge CLK)

		// On the second clock cycle FRAME_N remains asserted, CBE is set to allow all bytes to be read

		// The value of the first word to be written is placed on the bus

		// IRDY_N is asserted to indicate that the initiator is ready to transmit

		reg_frame_n <= 1'b0;

		reg_cbe <= 4'b0000;

		reg_ad <= START_DATA;

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;

		reg_irdy_n <= 1'b0;

      tXFER_STATUS_S2PCI ("1111", status_code);

      #1

      if (status_code == 4'b0000 || status_code == 4'b0010)

         $display ("Memory write #1 at Addr %h Data = %h", START_ADDRESS, START_DATA);

	  //Now write out the remainder of the data

	  if ((number_of_items != 1) && (status_code == 4'b000))		// If there is more than one item of data and the first write was sucessful

		  begin

		  while ((status_code == 4'b0000 || status_code == 4'b001) && word_count < number_of_items) // While there is data to write and the target has not disconnected

		  	begin

			// If this is the second last data item, disassert FRAME_N

			if (word_count == (number_of_items - 1))

				reg_frame_n = 1'b1;





			reg_ad = START_DATA + word_count;

			@(posedge CLK)

			tXFER_STATUS_S2PCI("1111", status_code);

			#1

			if (status_code == 4'b0000 || status_code == 4'b0010)

         		$display ("Memory write #%d at Addr %h Data = %h",word_count, START_ADDRESS, START_DATA + word_count);



			word_count = word_count + 1;

		    end

		end



      // After the write has taken place, IRDY_N is deasserte, FRAME_N and CBE are tristated

      reg_irdy_n = 1'b1;

      reg_frame_n = 1'b1;

      ad_oe = 1'b0;

      cbe_oe = 1'b0;

      @(posedge CLK)



      init_oe <= 1'b0;

      @(posedge CLK);

	  RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase



   end

endtask //tMEMORY_WRITE_N





task tMEMORY_WRITE_1;

   input [31:0] START_ADDRESS;

   input [31:0] DATA;

   output [7:0] RETURN_STATUS;



   begin



   tMEMORY_WRITE_N (START_ADDRESS, DATA, 1, RETURN_STATUS);



   end

endtask // tMEMORY_WRITE_1







task tMEMORY_WRITE_8_CBE;

	input [31:0] 	START_ADDRESS;

	input [31:0] 	DATA_0;

	input [31:0] 	DATA_1;

	input [31:0] 	DATA_2;

	input [31:0] 	DATA_3;

	input [31:0] 	DATA_4;

	input [31:0] 	DATA_5;

	input [31:0]	DATA_6;

	input [31:0] 	DATA_7;

   input [3:0]    CBE_0;

   input [3:0]    CBE_1;

   input [3:0]    CBE_2;

   input [3:0]    CBE_3;

   input [3:0]    CBE_4;

   input [3:0]    CBE_5;

   input [3:0]    CBE_6;

   input [3:0]    CBE_7;

	output [7:0]    RETURN_STATUS;



	reg [3:0]      	status_code;

   reg [31:0]     	data [7:0];

   reg [3:0]      byte_enable [7:0];

	integer			number_of_items;

	integer			word_count;   // Then number of words that have been written



	begin

		$display ($time, " ns, Beginning write to address %hH", START_ADDRESS);

		number_of_items = 8; // For this task the number of items is fixed

		word_count = 1;

		data[0] = DATA_0;

		data[1] = DATA_1;

		data[2] = DATA_2;

		data[3] = DATA_3;

		data[4] = DATA_4;

		data[5] = DATA_5;

		data[6] = DATA_6;

		data[7] = DATA_7;



      byte_enable [0] = CBE_0;

      byte_enable [1] = CBE_1;

      byte_enable [2] = CBE_2;

      byte_enable [3] = CBE_3;

      byte_enable [4] = CBE_4;

      byte_enable [5] = CBE_5;

      byte_enable [6] = CBE_6;

      byte_enable [7] = CBE_7;

		@(posedge CLK) // Make sure everthing is synchronous



		// On the first clock cycle FRAME_N is asserted, the address is put on the AD bus and

		// the command for memory write is placed on CBE

		reg_frame_n <= 1'b0;

		reg_irdy_n <= 1'b1;

		reg_ad <= START_ADDRESS;

		reg_cbe <= 4'b0111;

		init_oe <= 1'b1;

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;



		@(posedge CLK)

		// On the second clock cycle FRAME_N remains asserted, CBE is set to allow all bytes to be read

		// The value of the first word to be written is placed on the bus

		// IRDY_N is asserted to indicate that the initiator is ready to transmit

		reg_frame_n <= 1'b0;

		reg_cbe <= byte_enable[0];

		reg_ad <= data[0];

		ad_oe <= 1'b1;

		cbe_oe <= 1'b1;

		reg_irdy_n <= 1'b0;

      tXFER_STATUS_S2PCI ("1111", status_code);

      #1

      if (status_code == 4'b0000 || status_code == 4'b0010)

         $display ("Memory write #1 at Addr %h Data = %h", START_ADDRESS, DATA_0);

	  //Now write out the remainder of the data

	  if ((number_of_items != 1) && (status_code == 4'b000))		// If there is more than one item of data and the first write was sucessful

		  begin

		  while ((status_code == 4'b0000 || status_code == 4'b001) && word_count < number_of_items) // While there is data to write and the target has not disconnected

		  	begin

			// If this is the second last data item, disassert FRAME_N

			if (word_count == (number_of_items - 1))

				reg_frame_n = 1'b1;





			reg_ad <= data[word_count];

         reg_cbe <= byte_enable [word_count];

			@(posedge CLK)

			tXFER_STATUS_S2PCI("1111", status_code);

			#1

			if (status_code == 4'b0000 || status_code == 4'b0010)

         		$display ("Memory write #%d at Addr %h Data = %h",word_count, START_ADDRESS, data[word_count]);



			word_count = word_count + 1;

		    end

		end



      // After the write has taken place, IRDY_N is deasserte, FRAME_N and CBE are tristated

      reg_irdy_n = 1'b1;

      reg_frame_n = 1'b1;

      ad_oe = 1'b0;

      cbe_oe = 1'b0;

      @(posedge CLK)



      init_oe <= 1'b0;

      @(posedge CLK);

	  RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase



   end

endtask //tMEMORY_WRITE_8_CBE



task tMEMORY_READ_8;



	input [31:0] 	START_ADDRESS;

	output [31:0]	DATA_0;

	output [31:0]	DATA_1;

	output [31:0]	DATA_2;

	output [31:0]	DATA_3;

	output [31:0]	DATA_4;

	output [31:0]	DATA_5;

	output [31:0]	DATA_6;

	output [31:0]	DATA_7;



	output [7:0]	RETURN_STATUS;



	reg [3:0]		status_code;

	reg [31:0]		data[7:0];

	integer			number_of_items;

	integer			word_count;



	begin



	$display ($time, " ns, Beginning read from address = %hH", START_ADDRESS);

	number_of_items = 8;		// This task will always try and read 8 items

	word_count = 1;



	@(posedge CLK)

	// On the first Clock, FRAME_N is asserted, the address is put on the AD bus and the command

	// for memory write is placed on the CBE

	reg_frame_n = 1'b0;

	reg_irdy_n = 1'b1;

	reg_ad = START_ADDRESS;

	reg_cbe = 4'b0110;

	init_oe = 1'b1;

	ad_oe = 1'b1;

	cbe_oe = 1'b1;



	@(posedge CLK)

	// On the second clock cycle FRAME_N remains asserted (unless it is a single item READ) CBE is

	// set to allow all bytes to be read. AD is tri-stated to allow the target to place data on the AD

	// bus and IRDY_N is asserted to indicate the initiator is ready to recieve data

	if (number_of_items == 1)

		reg_frame_n = 1'b1;

	

	reg_irdy_n = 1'b0;

	reg_cbe = 1'b0;

	init_oe = 1'b1;

	ad_oe = 1'b0;

	cbe_oe = 1'b1;

	tXFER_STATUS_S2PCI (4'b0000, status_code);

	data [word_count - 1] = AD;

	#1

	if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

		$display ("Memory Read #%d at ADDR = %hH Data = %hH", word_count, START_ADDRESS, data[word_count - 1]);

	// If there is more than one item to be read and the first read was successful then carry on reading

	if (number_of_items != 1 && status_code == 4'b0000)

		begin

		while ((status_code == 4'b000 || status_code == 4'b0001) && word_count < number_of_items)

			begin

			if (word_count == number_of_items - 1)  // Deassert frame on the second last item

				reg_frame_n = 1'b1;



			@(posedge CLK)

			tXFER_STATUS_S2PCI(4'b0000, status_code);

			data[word_count] = AD;

			#1

			if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

				$display ("Memory Read #%d at ADDR = %hH Data = %hH", (word_count + 1), (START_ADDRESS + 4 * word_count), data[word_count]);



			word_count = word_count + 1;

			end

		end

	// After the read is finished, IRDY is deasserted

	reg_irdy_n <= 1'b1;

	@(posedge CLK)

	// And then all PCI signals are tristated

	init_oe <= 1'b0;

	cbe_oe <= 1'b0;



	@(posedge CLK);

	RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase

   DATA_0 = data[0];

   DATA_1 = data[1];

   DATA_2 = data[2];

   DATA_3 = data[3];

   DATA_4 = data[4];

   DATA_5 = data[5];

   DATA_6 = data[6];

   DATA_7 = data[7];



	end

endtask // tMEMORY_READ_8





// This will attempt to read a number of data items equal to BURST_SIZE

// It will only return the first 4 data items read

task tMEMORY_READ_N;



	input [31:0] 	START_ADDRESS;

   input [31:0]   BURST_SIZE;

	output [31:0]	DATA_0;

	output [31:0]	DATA_1;

	output [31:0]	DATA_2;

	output [31:0]	DATA_3;

	output [31:0]	DATA_4;

	output [31:0]	DATA_5;

	output [31:0]	DATA_6;

	output [31:0]	DATA_7;



	output [7:0]	RETURN_STATUS;



	reg [3:0]		status_code;

	reg [31:0]		data[7:0];

	integer			number_of_items;

	integer			word_count;



	begin



	$display ($time, " ns, Beginning read from address = %hH", START_ADDRESS);

	number_of_items = BURST_SIZE;		// This task will always try and read 8 items

	word_count = 1;



	@(posedge CLK)

	// On the first Clock, FRAME_N is asserted, the address is put on the AD bus and the command

	// for memory write is placed on the CBE

	reg_frame_n = 1'b0;

	reg_irdy_n = 1'b1;

	reg_ad = START_ADDRESS;

	reg_cbe = 4'b0110;

	init_oe = 1'b1;

	ad_oe = 1'b1;

	cbe_oe = 1'b1;



	@(posedge CLK)

	// On the second clock cycle FRAME_N remains asserted (unless it is a single item READ) CBE is

	// set to allow all bytes to be read. AD is tri-stated to allow the target to place data on the AD

	// bus and IRDY_N is asserted to indicate the initiator is ready to recieve data

	if (number_of_items == 1)

		reg_frame_n = 1'b1;



	reg_irdy_n = 1'b0;

	reg_cbe = 1'b0;

	init_oe = 1'b1;

	ad_oe = 1'b0;

	cbe_oe = 1'b1;

	tXFER_STATUS_S2PCI (4'b0000, status_code);

	data [word_count - 1] = AD;

	#1

	if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

		$display ("Memory Read #%d at ADDR = %hH Data = %hH", word_count, START_ADDRESS, data[word_count - 1]);

	// If there is more than one item to be read and the first read was successful then carry on reading

	if (number_of_items != 1 && status_code == 4'b0000)

		begin

		while ((status_code == 4'b000 || status_code == 4'b0001) && word_count < number_of_items)

			begin

			if (word_count == number_of_items - 1)  // Deassert frame on the second last item

				reg_frame_n = 1'b1;



			@(posedge CLK)

			tXFER_STATUS_S2PCI(4'b0000, status_code);

			data[word_count] = AD;

			#1

			if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

				$display ("Memory Read #%d at ADDR = %hH Data = %hH", (word_count + 1), (START_ADDRESS + 4 * word_count), data[word_count]);



			word_count = word_count + 1;

			end

		end

	// After the read is finished, IRDY is deasserted

	reg_irdy_n <= 1'b1;

	@(posedge CLK)

	// And then all PCI signals are tristated

	init_oe <= 1'b0;

	cbe_oe <= 1'b0;



	@(posedge CLK);

	RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase

   DATA_0 = data[0];

   DATA_1 = data[1];

   DATA_2 = data[2];

   DATA_3 = data[3];

   DATA_4 = data[4];

   DATA_5 = data[5];

   DATA_6 = data[6];

   DATA_7 = data[7];



	end

endtask // tMEMORY_READ_N



task tMEMORY_READ_1; // This task will read a single data phase

   input [31:0]   START_ADDRESS;

   output [31:0]  DATA;

   output [7:0]   RETURN_STATUS;





   reg [31:0]	data_0;

   reg [31:0]	data_1;

   reg [31:0]	data_2;

   reg [31:0]	data_3;

   reg [31:0]	data_4;

   reg [31:0]	data_5;

   reg [31:0]	data_6;

   reg [31:0]	data_7;



   reg [7:0] status;

   reg [31:0] burst_size;



   begin



   tMEMORY_READ_N(START_ADDRESS, 1, DATA, data_1, data_2, data_3, data_4, data_5, data_6, data_7, RETURN_STATUS);



   end

endtask // tMEMORY_READ_N





task tMEMORY_READ_8_WAIT;  // This task is like the regular read 8 but adds wait states



	input [31:0] 	START_ADDRESS;

	output [31:0]	DATA_0;

	output [31:0]	DATA_1;

	output [31:0]	DATA_2;

	output [31:0]	DATA_3;

	output [31:0]	DATA_4;

	output [31:0]	DATA_5;

	output [31:0]	DATA_6;

	output [31:0]	DATA_7;



	output [7:0]	RETURN_STATUS;



	reg [3:0]		status_code;

	reg [31:0]		data[7:0];

	integer			number_of_items;

	integer			word_count;



	begin



	$display ($time, " ns, Beginning read from address = %hH", START_ADDRESS);

	number_of_items = 8;		// This task will always try and read 8 items

	word_count = 1;



	@(posedge CLK)

	// On the first Clock, FRAME_N is asserted, the address is put on the AD bus and the command

	// for memory write is placed on the CBE

	reg_frame_n = 1'b0;

	reg_irdy_n = 1'b1;

	reg_ad = START_ADDRESS;

	reg_cbe = 4'b0110;

	init_oe = 1'b1;

	ad_oe = 1'b1;

	cbe_oe = 1'b1;



	@(posedge CLK)

	// On the second clock cycle FRAME_N remains asserted (unless it is a single item READ) CBE is

	// set to allow all bytes to be read. AD is tri-stated to allow the target to place data on the AD

	// bus and IRDY_N is asserted to indicate the initiator is ready to recieve data

	if (number_of_items == 1)

		reg_frame_n = 1'b1;



	reg_irdy_n = 1'b0;

	reg_cbe = 1'b0;

	init_oe = 1'b1;

	ad_oe = 1'b0;

	cbe_oe = 1'b1;

	tXFER_STATUS_S2PCI (4'b0000, status_code);

	data [word_count - 1] = AD;

	#1

	if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

		$display ("Memory Read #%d at ADDR = %hH Data = %hH", word_count, START_ADDRESS, data[word_count - 1]);

	// If there is more than one item to be read and the first read was successful then carry on reading

	if (number_of_items != 1 && status_code == 4'b0000)

		begin

      // Insert a wait state

      reg_irdy_n = 1'b1;

      @(posedge CLK)

      reg_irdy_n = 1'b0;

		while ((status_code == 4'b000 || status_code == 4'b0001) && word_count < number_of_items)

			begin

			if (word_count == number_of_items - 1)  // Deassert frame on the second last item

				reg_frame_n = 1'b1;



			@(posedge CLK)

			tXFER_STATUS_S2PCI(4'b0000, status_code);

			data[word_count] = AD;

			#1

			if (status_code == 4'b0000 || status_code == 4'b001 || status_code == 4'b0010)

				$display ("Memory Read #%d at ADDR = %hH Data = %hH", (word_count + 1), (START_ADDRESS + 4 * word_count), data[word_count]);



			word_count = word_count + 1;

			end

		end

	// After the read is finished, IRDY is deasserted

	reg_irdy_n <= 1'b1;

	@(posedge CLK)

	// And then all PCI signals are tristated

	init_oe <= 1'b0;

	cbe_oe <= 1'b0;



	@(posedge CLK);

	RETURN_STATUS [5:0] = word_count;

	  case (status_code)

		  4'b0000 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0001 : RETURN_STATUS [7:6] = 2'b00;

		  4'b0010 : RETURN_STATUS [7:6] = 2'b01;

		  4'b0011 : RETURN_STATUS [7:6] = 2'b10;

		  default : RETURN_STATUS [7:6] = 2'b11;

	  endcase

   DATA_0 = data[0];

   DATA_1 = data[1];

   DATA_2 = data[2];

   DATA_3 = data[3];

   DATA_4 = data[4];

   DATA_5 = data[5];

   DATA_6 = data[6];

   DATA_7 = data[7];



	end

endtask // tMEMORY_READ_8_WAIT





task tCHECK_DATA_8;



	input [31:0]   GOOD_DATA_0;

	input [31:0]   GOOD_DATA_1;

	input [31:0]   GOOD_DATA_2;

	input [31:0]   GOOD_DATA_3;

	input [31:0]   GOOD_DATA_4;

	input [31:0]   GOOD_DATA_5;

	input [31:0]   GOOD_DATA_6;

	input [31:0]   GOOD_DATA_7;



	input [31:0]   DATA_0;

	input [31:0]   DATA_1;

	input [31:0]   DATA_2;

	input [31:0]   DATA_3;

	input [31:0]   DATA_4;

	input [31:0]   DATA_5;

	input [31:0]   DATA_6;

	input [31:0]   DATA_7;



	output [7:0]   STATUS;



   begin

   STATUS[0] = (GOOD_DATA_0 === DATA_0);

   STATUS[1] = (GOOD_DATA_1 === DATA_1);

   STATUS[2] = (GOOD_DATA_2 === DATA_2);

   STATUS[3] = (GOOD_DATA_3 === DATA_3);

   STATUS[4] = (GOOD_DATA_4 === DATA_4);

   STATUS[5] = (GOOD_DATA_5 === DATA_5);

   STATUS[6] = (GOOD_DATA_6 === DATA_6);

   STATUS[7] = (GOOD_DATA_7 === DATA_7);



   end

endtask //tCHECK_DATA_8;











