//--------------------------------------------------------------------------------------------------

//					:

// File name	: PCI_DEVICE_INITIATOR_BUS.V

//					:

// Title			: Generic PCI device Initiator low level PCI bus Tasks

//					:

// Comments		:

//					:

//					:

// Conventions : - Port names are 'UPPER' case.

//					: - Internal signals and variables are 'lower' case.

//					: - Active low signals are identified with '_n' or '_N'.

//					: - instances begin with lower case 'i', rest upper case

//					: - parameters begin with lower case 'p', rest upper case

//					: - tasks begin with lower case 't', rest upper case

//					:

//					: Tab setting = 3

//					: Right Margin = 100

//					:

// Limitations : For use in testbench only

//					:

// Developers  : Majid Kalsi

//					:

//--------------------------------------------------------------------------------------------------

// Revision History :

//--------------------------------------------------------------------------------------------------

//	  Version No | Author	 | Changes Made:															| Mod. Date:

//		 v1.0		 | mk			 | First write																| 00/00/00

//--------------------------------------------------------------------------------------------------





task tSAC_SINGLE;

	// parameter declarations

	parameter pDATA_WIDTH = `BIT32;



	// input/output declarations

	input [3 : 0] TYPE_ACCESS;

	input [(pDATA_WIDTH*4-1) :0] BYTE_ENABLES;

	input [31 : 0] ADDR;

	inout [(pDATA_WIDTH*32-1) :0] DATA;



	// Variable declarations

	integer error;

	integer count;

   integer status;

   integer re_send_upper;

   reg [(pDATA_WIDTH*32-1) :0] temp;



	begin

		// do some error checking

		error = tERR_CHK_SAC_SINGLE(TYPE_ACCESS, pDATA_WIDTH);



		if(error == `FALSE) begin

			// assert request for bus, and wait for grant

			reg_req_n = 1'b0;

			while(GNT_N == 1)

         begin

				@(posedge CLK);



         end

			reg_req_n = 1'b1;             // release request



			// address phase

			reg_frame_n = 1'b0;

			reg_req64_n = (pDATA_WIDTH == `BIT64) ? 1'b0 : 1'b1;

			reg_ad = ADDR;

			reg_cbe = TYPE_ACCESS;

			init_oe = 1'b1;               // drive initiator signals

			ad_oe = 1'b1;                 // drive AD

			cbe_oe = 1'b1;                // drive CBE



			// at next clk

			// wait IRDY initial latency, then drive IRDY

			@(posedge CLK);

			ad_oe = 1'b0;        			// tri-state AD

			cbe_oe = 1'b0;       			// tri-state CBE

			count = 0;

			while(count != IRDY_INITIAL_LATENCY) begin

				 @(posedge CLK);

				 count = count + 1;

			end

			reg_irdy_n = 1'b0;            // drive IRDY

			reg_cbe = BYTE_ENABLES;

			cbe_oe = 1'b1;                // drive CBE

			if(tIS_WRITE(TYPE_ACCESS)) begin

				reg_ad = DATA;

				ad_oe = 1'b1;              // drive AD

			end



         // de-assert frame

			reg_frame_n = 1'b1;

			reg_req64_n = 1'b1;



			if(TYPE_ACCESS == `SPECIAL) begin

				// drive IRDY for 5 clks

			  	for(count = 0; count < 5; count = count + 1) begin

					// Special Cycle data is only driven for one data phase

					if(count == 1) begin

                  ad_oe = 1'b0;        // tri-state AD

                  cbe_oe= 1'b0;        // tri-state CBE

               end



					// Wait until rising edge

					@(posedge CLK);

				end

			end

			else begin

           	// check transfer, if target not ready

            re_send_upper = `FALSE;

            if(!DEVSEL_N && !TRDY_N) begin

               @(posedge CLK);

					#(pDATA_WIDTH) tXFER_STATUS(DATA, status);

            end

            else

					#(pDATA_WIDTH) tXFER_STATUS(DATA, status);



				// check if 32 bit target

            if((pDATA_WIDTH == `BIT64) && (status == `TERM_NORM32)) begin

               // transfer upper 32 bit data

					reg_cbe ={{4{1'bZ}}, BYTE_ENABLES[7:4]};



					if(tIS_WRITE(TYPE_ACCESS)) begin

						reg_ad = {{32{1'bZ}}, DATA[63:32]};

						ad_oe = 1'b1;        // drive AD

					end



					// check transfer

					#(`BIT32) tXFER_STATUS(DATA[63:32], status);

               if(status == `TERM_DWOD)

                  re_send_upper = `TRUE;

            end

			end



         // de-assert IRDY

         reg_irdy_n = 1'b1;

         ad_oe = 1'b0;

         cbe_oe = 1'b0;

         @(posedge CLK);

         init_oe = 1'b0;



         if(status == `TERM_DWOD) begin

				tINTERCYCLE_GAP;

            if(re_send_upper)

            	#(`BIT32) tSAC_SINGLE(TYPE_ACCESS, BYTE_ENABLES[7:4], ADDR, DATA[63:32]);

            else

            	#(pDATA_WIDTH) tSAC_SINGLE(TYPE_ACCESS, BYTE_ENABLES, ADDR, DATA);

            // end if

         end

		end

	end

endtask











task tXFER_STATUS;

	// parameter declarations

	parameter pDATA_WIDTH = `BIT32;



	// input/output declarations

	output [(pDATA_WIDTH*32-1) :0] RETURN_DATA;

	output [3:0] RETURN_STAT;



	// Variable declarations

	integer devsel_cnt;

	integer trdy_cnt;



	begin

		devsel_cnt = 0;

		trdy_cnt = 0;



		// Check for Target Abort

		if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 1) begin

			RETURN_DATA = 'bx;

			RETURN_STAT = 4;

		end

		else begin

			// wait for DEVSEL

			while(DEVSEL_N && (devsel_cnt < 5)) begin

				@(posedge CLK);

				devsel_cnt = devsel_cnt + 1;

			end



			// wait for TRDY

			while(TRDY_N  && STOP_N && ((trdy_cnt < 16) && (devsel_cnt < 5))) begin

				trdy_cnt = trdy_cnt + 1;

				@(posedge CLK);

			end



			if (devsel_cnt < 5) begin

				if (trdy_cnt <= 16) begin

					if (TRDY_N == 0 && STOP_N == 1) begin

						RETURN_DATA = AD;

                  if(ACK64_N == 0)

							RETURN_STAT = `TERM_NORM64;

                  else

							RETURN_STAT = `TERM_NORM32;

					end

					else if(TRDY_N == 0 && STOP_N == 0) begin

						RETURN_DATA = AD;

                  if(ACK64_N == 0)

							RETURN_STAT = `TERM_DWD64;

                  else

							RETURN_STAT = `TERM_DWD32;

					end

					else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 0) begin

						RETURN_DATA = 'bx;

						RETURN_STAT = `TERM_DWOD;

					end

					else if (TRDY_N==1 && STOP_N == 0 && DEVSEL_N == 1) begin

						RETURN_DATA = 'bx;

						RETURN_STAT = `TERM_TABORT;

					end

					else if (TRDY_N==1 && STOP_N == 1) begin

						$display("[tXFER_STATUS] ERROR: Check Transfer Procedure");

						RETURN_DATA = 'bx;

						RETURN_STAT = `TERM_ERROR;

					end

				end

				else begin

					$display("[tXFER_STATUS] ERROR: No Target Response");

					RETURN_DATA = 'bx;

					RETURN_STAT = `TERM_ERROR;

				end

         end

			else begin

				RETURN_DATA = 'bx;

				RETURN_STAT = `TERM_MABORT;

			end

		end

	end

endtask





task tINTERCYCLE_GAP;

	begin

		@(posedge CLK);

		@(posedge CLK);

		@(posedge CLK);

		@(posedge CLK);

	end

endtask





function tERR_CHK_SAC_SINGLE;

	// input/output declarations

	input TYPE_ACCESS;

	input DATA_WIDTH;



	// Variable declarations

	reg	error;



	begin

		error = `FALSE;



		case(TYPE_ACCESS)

			`INT_ACK :

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("Interrupt Acknowledge Cycle cannot bit 64 bit");

						error = `TRUE;

					end

				end



			`SPECIAL :

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("Special Cycle cannot bit 64 bit");

						error = `TRUE;

					end

            end



			`IO_RD	:

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("IO Read Cycle cannot bit 64 bit");

						error = `TRUE;

					end

				end



			`IO_WR	:

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("IO Write Cycle cannot bit 64 bit");

						error = `TRUE;

					end

				end



			`CFG_RD  :

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("Configuration Read Cycle cannot bit 64 bit");

						error = `TRUE;

					end

				end



			`CFG_WR  :

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("Configuration Write Cycle cannot bit 64 bit");

						error = `TRUE;

					end

				end



			`MWI  	:

				begin

					if(!cfg_cmd_reg[4]) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("Memory Write & Invalidate not enabled in Configuration Command Register");

						error = `TRUE;

					end

				end



			`DAC		:

				begin

					if(DATA_WIDTH == `BIT64) begin

						$display($time, "[%s INITIATOR] Invalid use of tSAC_SINGLE access", pINST_NAME);

						$display("For DAC cycle, use tDAC_SINGLE or tDAC_BURST");

						error = `TRUE;

					end

				end

			default : ;// do nothing

		endcase



		tERR_CHK_SAC_SINGLE = error;

	end

endfunction





function tIS_READ;

	// input/output declarations

	input TYPE_ACCESS;



	begin

		case(TYPE_ACCESS)

			`INT_ACK :  tIS_READ = `TRUE;

			`IO_RD	:  tIS_READ = `TRUE;

			`MEM_RD  :  tIS_READ = `TRUE;

			`CFG_RD  :  tIS_READ = `TRUE;

			`MRM		:  tIS_READ = `TRUE;

			`MRL		:  tIS_READ = `TRUE;

			default  :  tIS_READ = `FALSE;

		endcase

	end

endfunction





function tIS_WRITE;

	// input/output declarations

	input TYPE_ACCESS;



	begin

		case(TYPE_ACCESS)

			`SPECIAL :  tIS_WRITE = `TRUE;

			`IO_WR	:  tIS_WRITE = `TRUE;

			`MEM_WR  :  tIS_WRITE = `TRUE;

			`CFG_WR  :  tIS_WRITE = `TRUE;

			`MWI		:  tIS_WRITE = `TRUE;

			default  :  tIS_WRITE = `FALSE;

		endcase

	end

endfunction









