#define GUID_DRIVER

#include "sii200.h"



NTSTATUS

PnPStartDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



NTSTATUS

PnPStopDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



NTSTATUS

PnPRemoveDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );



NTSTATUS

CompletionRoutine (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp,

  IN PVOID              Context

  );



NTSTATUS

SendIrpSync (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  );





// IRQL PASSIVE_LEVEL

NTSTATUS

AddDevice (

  IN PDRIVER_OBJECT     DriverObject,

  IN PDEVICE_OBJECT     PhysicalDeviceObject

  )

{

	NTSTATUS              status = STATUS_SUCCESS;

	PDEVICE_OBJECT        deviceObject = NULL;

	PDEVICE_EXTENSION     deviceExtension;



	PAGED_CODE ();



	KdPrint (("sii200.sys: AddDevice\n"));



	// create device object

	status = IoCreateDevice (

      DriverObject,

      sizeof (DEVICE_EXTENSION),

      NULL,

      FILE_DEVICE_UNKNOWN,

      0,

      FALSE,

      &deviceObject);



	if (!NT_SUCCESS (status))  

	{

		KdPrint (("sii200.sys: IoCreateDevice exits with 0x%x\n", status));

		return status;

	}



	LogMsg(deviceObject, L"SpartanII-200 device created successfully!");

	deviceExtension = (PDEVICE_EXTENSION) deviceObject->DeviceExtension;



	// create symbolic link

	status = IoRegisterDeviceInterface (

      PhysicalDeviceObject,

      (LPGUID) &GuidDriver,

      NULL,

      &deviceExtension->InterfaceName);



	if (!NT_SUCCESS (status))  

	{

      KdPrint (("sii200.sys: IoRegisterDeviceInterface exits with 0x%x\n", status));

      IoDeleteDevice (deviceObject);

      return status;

	}



	// initialize driver extension

	deviceExtension->PhysicalDeviceObject = PhysicalDeviceObject;

	deviceExtension->FunctionDeviceObject = deviceObject;

	deviceExtension->bar = 0;



	INITIALIZE_PNP_STATE (deviceExtension);

	// set power management settings



	// initialize device objects



	// attach to driver stack

	deviceExtension->NextStackDeviceObject = IoAttachDeviceToDeviceStack (

      deviceObject,

      PhysicalDeviceObject);



	// set flags

    deviceObject->Flags |= DO_BUFFERED_IO;

    //deviceObject->Flags |= DO_DIRECT_IO;

	deviceObject->Flags &= ~DO_DEVICE_INITIALIZING;



	KdPrint (("sii200.sys: AddDevice complete\n"));



	return status;

} // AddDevice



// IRQL PASSIVE_LEVEL

NTSTATUS

DispatchPnP (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	NTSTATUS              status = STATUS_SUCCESS;

	PDEVICE_EXTENSION     deviceExtension;

	PIO_STACK_LOCATION    ioStack;



	PAGED_CODE ();



	KdPrint (("sii200.sys:  DispatchPnP\n"));



	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;

	ioStack = IoGetCurrentIrpStackLocation (Irp);



	status = SendIrpSync (deviceExtension->NextStackDeviceObject, Irp);



	if (!NT_SUCCESS (status))  

	{

		KdPrint (("SendIrp exits with 0x%x", status));

		IoCompleteRequest (Irp, IO_NO_INCREMENT);

		return status;

	}



	switch (ioStack->MinorFunction)  {

  

	case IRP_MN_START_DEVICE:

		LogMsg(DeviceObject, L"Device started!");

		status = PnPStartDevice (DeviceObject, Irp);

		break;



	case IRP_MN_QUERY_STOP_DEVICE:

		LogMsg(DeviceObject, L"Device received QueryStopDevice!");

		SET_NEW_PNP_STATE (deviceExtension, StopPending);

		break;



	case IRP_MN_CANCEL_STOP_DEVICE:

		LogMsg(DeviceObject, L"Device received CancelStopDevice!");

		RESTORE_PREVIOUS_PNP_STATE (deviceExtension);

		break;



	case IRP_MN_STOP_DEVICE:

		LogMsg(DeviceObject, L"Device stopped!");

		status = PnPStopDevice (DeviceObject, Irp);

		break;



	case IRP_MN_QUERY_REMOVE_DEVICE:

		LogMsg(DeviceObject, L"Device received QueryRemoveDevice!");

		SET_NEW_PNP_STATE (deviceExtension, RemovePending);

		break;



	case IRP_MN_CANCEL_REMOVE_DEVICE:

		LogMsg(DeviceObject, L"Device received CancelRemoveDevice!");

		RESTORE_PREVIOUS_PNP_STATE (deviceExtension);

		break;



	case IRP_MN_SURPRISE_REMOVAL:

		LogMsg(DeviceObject, L"Device received SurpriseRemoval!");

		SET_NEW_PNP_STATE (deviceExtension, SurpriseRemovePending);

		break;



	case IRP_MN_REMOVE_DEVICE:

		LogMsg(DeviceObject, L"Device received RemoveDevice!");

		status = PnPRemoveDevice (DeviceObject, Irp);

		break;

	}



	KdPrint (("sii200.sys: DispatchPnP complete\n"));



	return status;

} // DispatchPnP





NTSTATUS

PnPStartDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	PDEVICE_EXTENSION				deviceExtension;

	ULONG							i;

	ULONG							idx;

	PIO_STACK_LOCATION				ioStack;

	PCM_PARTIAL_RESOURCE_DESCRIPTOR rawResource;

	PCM_PARTIAL_RESOURCE_LIST       rawResourceList;

	NTSTATUS						status = STATUS_SUCCESS;

	PCM_PARTIAL_RESOURCE_DESCRIPTOR translatedResource;

	PCM_PARTIAL_RESOURCE_LIST       translatedResourceList;



	PAGED_CODE ();



	KdPrint (("sii200.sys: PnPStartDevice\n"));



	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;

	ioStack = IoGetCurrentIrpStackLocation (Irp);



	if (ioStack->Parameters.StartDevice.AllocatedResources &&

      ioStack->Parameters.StartDevice.AllocatedResourcesTranslated)  

	{

		rawResourceList = 

		  &ioStack->Parameters.StartDevice.AllocatedResources->List[0].PartialResourceList;

		rawResource = &rawResourceList->PartialDescriptors[0];



		translatedResourceList = 

		  &ioStack->Parameters.StartDevice.AllocatedResourcesTranslated->List[0].PartialResourceList;

		translatedResource = &translatedResourceList->PartialDescriptors[0];



		for (i = 0; 

			 i < rawResourceList->Count; 

			 i++, rawResource++, translatedResource++)  

		{

			switch (rawResource->Type)  

			{

			case CmResourceTypeNull:

				KdPrint (("    Resource RAW Null"));

				break;



			case CmResourceTypePort:

				KdPrint (("    Resource Raw Port: (0x%x) Length: (0x%x)",

				  rawResource->u.Port.Start.LowPart, rawResource->u.Port.Length));



				switch (translatedResource->Type)  

				{

				case CmResourceTypePort:

					KdPrint (("    Resource Translated Port: (0x%x) Length: (0x%x)",

					  translatedResource->u.Port.Start.LowPart, 

					  translatedResource->u.Port.Length));

					break;



				case CmResourceTypeMemory:

					KdPrint (("    Resource Translated Memory: (0x%x) Length: (0x%x)",

					  translatedResource->u.Memory.Start.LowPart, 

					  translatedResource->u.Memory.Length));

					break;



				default:

					KdPrint (("    Unhandled resource 0x%x", translatedResource->Type));

					break;

				}



				break;



			case CmResourceTypeInterrupt:

				KdPrint (("    Resource Interrupt"));

				break;



			case CmResourceTypeMemory:

				KdPrint (("    Resource Raw Memory: (0x%x) Length: (0x%x)",

				  rawResource->u.Memory.Start.LowPart, 

				  rawResource->u.Memory.Length));

				KdPrint (("    Resource Translated Memory: (0x%x) Length: (0x%x)",

				  translatedResource->u.Memory.Start.LowPart, 

				  translatedResource->u.Memory.Length));



				// map memory

				idx = deviceExtension->bar;

				deviceExtension->MemoryOffset[idx] = 0;

				deviceExtension->MemoryLength[idx] = translatedResource->u.Memory.Length;

				deviceExtension->MemoryStart[idx]  = MmMapIoSpace (

				  translatedResource->u.Memory.Start,

				  translatedResource->u.Memory.Length,

				  MmNonCached);

				deviceExtension->Memory[idx] = translatedResource->u.Memory.Start;

				KdPrint (("Translated resource is memory at 0x%x", 

				  deviceExtension->MemoryStart));



				if (idx < 1)

					deviceExtension->bar += 1;



				break;



			case CmResourceTypeDma:

				break;



			case CmResourceTypeDeviceSpecific:

				break;



			default:

				break;

			}

		}

	}

  

	// enable interface

	IoSetDeviceInterfaceState (&deviceExtension->InterfaceName, TRUE);



	SET_NEW_PNP_STATE (deviceExtension, Started);



	KdPrint (("sii200.sys: PnPStartDevice complete\n"));



	return status;

} // PnPStartDevice





NTSTATUS

PnPStopDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	NTSTATUS              status = STATUS_SUCCESS;

	PDEVICE_EXTENSION     deviceExtension;

	ULONG				  idx;



	PAGED_CODE ();



	KdPrint (("sii200.sys: PnPStopDevice\n"));



	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;



	// unmap memory

	for (idx = 0; idx < deviceExtension->bar; idx++)

		if (deviceExtension->MemoryStart[idx])

			MmUnmapIoSpace (deviceExtension->MemoryStart[idx],

			  deviceExtension->MemoryLength[idx]);



	// disable interface

	IoSetDeviceInterfaceState (&deviceExtension->InterfaceName, FALSE);



	SET_NEW_PNP_STATE (deviceExtension, Stopped);



	// Record the action in the Event Log.

	LogMsg(DeviceObject, L"SpartanII-200 device driver stopped");



	KdPrint (("sii200.sys: PnPStopDevice complete\n"));



	return status;

} // PnPStopDevice





NTSTATUS

PnPRemoveDevice (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	NTSTATUS              status = STATUS_SUCCESS;

	PDEVICE_EXTENSION     deviceExtension;



	PAGED_CODE ();



	KdPrint (("sii200.sys: PnPRemoveDevice\n"));



	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;



	SET_NEW_PNP_STATE (deviceExtension, Deleted);



	// remove device from stack

	if (deviceExtension->NextStackDeviceObject)

		IoDetachDevice (deviceExtension->NextStackDeviceObject);



	// free DeviceExtension

	RtlFreeUnicodeString (&deviceExtension->InterfaceName);



	// delete device

	LogMsg(DeviceObject, L"SpartanII-200 device removed successfully!");

	IoDeleteDevice (DeviceObject);



	KdPrint (("sii200.sys: PnPRemoveDevice complete\n"));



	return status;

} // PnPRemoveDevice





// IRQL PASSIVE_LEVEL

NTSTATUS

DispatchPower (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	NTSTATUS              status = STATUS_SUCCESS;

	PDEVICE_EXTENSION     deviceExtension;



	KdPrint (("sii200.sys: DispatchPower\n"));



	deviceExtension = (PDEVICE_EXTENSION) DeviceObject->DeviceExtension;



	// pass IRP down the stack

	PoStartNextPowerIrp (Irp);

	IoSkipCurrentIrpStackLocation (Irp);

	status = PoCallDriver (deviceExtension->NextStackDeviceObject, Irp);



	KdPrint (("sii200.sys: DispatchPower complete\n"));

  

	return status;

} // DispatchPower





NTSTATUS

CompletionRoutine (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp,

  IN PVOID              Context

  )

{

	PKEVENT event = (PKEVENT) Context;

	KeSetEvent (event, IO_NO_INCREMENT, FALSE);



	return STATUS_MORE_PROCESSING_REQUIRED;

} // CompletionRoutine





// IRQL

NTSTATUS

SendIrpSync (

  IN PDEVICE_OBJECT     DeviceObject,

  IN PIRP               Irp

  )

{

	KEVENT                event;

	NTSTATUS              status;



	PAGED_CODE ();



	KeInitializeEvent (&event, NotificationEvent, FALSE);

	IoCopyCurrentIrpStackLocationToNext (Irp);

	IoSetCompletionRoutine (

      Irp,

      CompletionRoutine,

      &event,

      TRUE,

      TRUE,

      TRUE);



	status = IoCallDriver (DeviceObject, Irp);



	if (status == STATUS_PENDING)  

	{

		KeWaitForSingleObject (

		  &event,

		  Executive,

		  KernelMode,

		  FALSE,

		  NULL);

		status = Irp->IoStatus.Status;

	}



	return status;

} // SendIrpSync

